@@ -8,7 +8,7 @@ import traceback
 from multiprocessing.synchronize import Event
 from typing import Callable, Type
 
-from .errors import ValidationError
+from .errors import DecodingError, ValidationError
 from .protocol import Protocol
 from .worker import Worker
 
@@ -57,6 +57,7 @@ class Coordinator:
         self.worker = worker()
         self.worker._set_mbs(max_batch_size)
         self.worker._set_stage(stage)
+        self.worker._set_id(worker_id)
 
         self.name = f"<{stage_id}|{worker.__name__}|{worker_id}>"
 
@@ -72,8 +73,8 @@ class Coordinator:
         self.shutdown = shutdown
         self.shutdown_notify = shutdown_notify
 
-        self.init_protocol()
         self.init_worker()
+        self.init_protocol()
         self.run()
 
     def exit(self):
@@ -160,15 +161,24 @@ class Coordinator:
                         "returned data doesn't match the input data:"
                         f"input({len(data)})!=output({len(payloads)})"
                     )
-            except ValidationError as err:
+            except DecodingError as err:
                 err_msg = str(err).replace("\n", " - ")
+                err_msg = (
+                    err_msg if len(err_msg) else "cannot deserialize request bytes"
+                )
+                logger.info(f"{self.name} decoding error: {err_msg}")
+                status = self.protocol.FLAG_BAD_REQUEST
+                payloads = (f"decoding error: {err_msg}".encode(),)
+            except ValidationError as err:
+                err_msg = str(err)
+                err_msg = err_msg if len(err_msg) else "invalid data format"
                 logger.info(f"{self.name} validation error: {err_msg}")
                 status = self.protocol.FLAG_VALIDATION_ERROR
-                payloads = (f"Validation Error: {err_msg}".encode(),)
+                payloads = (f"validation error: {err_msg}".encode(),)
             except Exception:
                 logger.warning(traceback.format_exc().replace("\n", " "))
                 status = self.protocol.FLAG_INTERNAL_ERROR
-                payloads = ("Internal Error".encode(),)
+                payloads = ("inference internal error".encode(),)
 
             try:
                 self.protocol.send(status, ids, payloads)
